@@ -3,9 +3,12 @@
 import os
 import glob
 import shutil
-from ytmdl import defaults, logger
+from html import unescape
 
-logger = logger.Logger("Dir")
+from ytmdl import defaults
+from simber import Logger
+
+logger = Logger("Dir")
 
 
 def cleanup(TRACK_INFO, index, datatype):
@@ -31,8 +34,11 @@ def cleanup(TRACK_INFO, index, datatype):
                 os.rename(SONG, name + '.mp3')
                 SONG_NAME = name + '.mp3'
                 SONG = SONG_NAME
+
         shutil.move(SONG, os.path.join(DIR, SONG_NAME))
 
+        _delete_cached_songs(datatype)
+
         logger.info('Moved to {}...'.format(DIR))
         return True
     except Exception as e:
@@ -40,22 +46,43 @@ def cleanup(TRACK_INFO, index, datatype):
         return False
 
 
+def _delete_cached_songs(ext='mp3'):
+    """Delete cached songs"""
+    # We need to call this after song is moved
+    # because otherwise if there is an error along the way
+    # next time a wrong song may be copied.
+    SONGS_PATH = os.path.join(
+        defaults.DEFAULT.SONG_TEMP_DIR,
+        '*{}'.format(ext)
+    )
+    deleted = False
+    for song in glob.glob(SONGS_PATH):
+        deleted = True
+        os.remove(song)
+        logger.debug('Removed "{}" from cache'.format(os.path.basename(song)))
+    if deleted:
+        logger.debug('{}'.format('Deleted cached songs'))
+
+
 def ret_proper_names(ordered_names):
     """Return a list with the names changed to itunespy supported ones.
 
     For eg: Artist to artist_name
     """
-    itunespy_dict = {'Artist': 'artist_name',
-                     'Title': 'track_name',
-                     'Album': 'collection_name',
-                     'Genre': 'primary_genre_name',
-                     'TrackNumber': 'track_number',
-                     'ReleaseDate': 'release_date'
-                     }
+    info_dict = {'Artist': 'artist_name',
+                 'Title': 'track_name',
+                 'Album': 'collection_name',
+                 'Genre': 'primary_genre_name',
+                 'TrackNumber': 'track_number',
+                 'ReleaseDate': 'release_date'
+                }
+
+    logger.debug(ordered_names)
+    logger.debug(info_dict)
 
     new_names = []
-    for names in ordered_names:
-        new_names.append(itunespy_dict[names])
+    for name in ordered_names:
+        new_names.append(info_dict.get(name))
 
     return new_names
 
@@ -112,15 +139,19 @@ def make_custom_dir(DIR, TRACK_INFO):
         last_element = None
         order_dir = ret_proper_names(order_dir)
 
-    # Convert TRACK_INFO
-    TRACK_INFO = vars(TRACK_INFO)
+    logger.debug(TRACK_INFO)
 
     if last_element is not None:
-        last_element = TRACK_INFO[last_element]
+        last_element = getattr(TRACK_INFO, last_element)
         order_dir = order_dir[:len(order_dir) - 1]
 
     for kw_name in order_dir:
-        dir_name = TRACK_INFO[kw_name]
+        dir_name = unescape(getattr(TRACK_INFO, kw_name))
+
+        # Sometimes, certain strings have / in the name which creates
+        # issues since those strings are used to create directories.
+        # Whenever there is a /, replace it with -
+        dir_name = dir_name.replace("/", "-")
 
         new_dir = os.path.join(base_DIR, dir_name)
 
