@@ -2,7 +2,7 @@
 
 import os
 import re
-from ytmdl.logger import Logger
+from simber import Logger
 from xdg.BaseDirectory import xdg_config_home
 
 
@@ -50,6 +50,19 @@ config_text = '''#*****************************************#
 # Supported values are 320 and 192
 #
 #QUALITY = "320"
+#
+#*****************************************#
+# The METADATA_PROVIDERS value is a comma separated
+# values that specifies wich API providers to use for getting
+# the song metadata. Available values right now are:
+# itunes, gaana, deezer, lastfm. saavn.
+# Please check the github page of ytmdl for more information.
+#
+#METADATA_PROVIDERS = "itunes, gaana"
+#*****************************************#
+# The DEFAULT_FORMAT denotes what to use as default between
+# m4a and mp3
+#DEFAULT_FORMAT = "mp3"
 #'''
 
 
@@ -75,6 +88,17 @@ class DEFAULTS:
         # The config path
         self.CONFIG_PATH = os.path.join(xdg_config_home, 'ytmdl')
 
+        # The default metadata providers
+        self.METADATA_PROVIDERS = ['itunes', 'gaana', 'saavn']
+
+        # The available metadata providers
+        self.AVAILABLE_METADATA_PROVIDERS = self.METADATA_PROVIDERS + \
+            ['deezer', 'lastfm']  # add new ones here
+
+        self.VALID_FORMATS = ['mp3', 'm4a', 'opus']
+
+        self.DEFAULT_FORMAT = 'mp3'
+
     def _get_music_dir(self):
         """Get the dir the file will be saved to."""
         # The first preference will be ~/Music.
@@ -176,7 +200,8 @@ def check_config_setup():
     return True
 
 
-def checkExistence(keyword, value):
+def checkValidity(keyword, value):
+
     """Check if the user specified value in config is possible."""
     if keyword == 'SONG_DIR':
         # In this case check if $ and -> are present
@@ -184,27 +209,41 @@ def checkExistence(keyword, value):
         if '$' in value:
             pos = value.find('$')
             value = value[:pos]
-
-        if os.path.isdir(value):
-            return True
-        else:
-            return False
+        return os.path.isdir(value)
     elif keyword == 'QUALITY':
         # Possible values that QUALITY can take
         possQ = ['320', '192']
-
-        if value in possQ:
-            return True
-        else:
+        return value in possQ
+    elif keyword == 'DEFAULT_FORMAT':
+        possF = DEFAULTS().VALID_FORMATS
+        return value in possF
+    elif keyword == 'METADATA_PROVIDERS':
+        # Possible values that METADATA_PROVIDERS can take
+        possM = DEFAULTS().AVAILABLE_METADATA_PROVIDERS
+        if not value:
+            logger.warning(
+                "Metadata provider value is empty. \
+                    Default values will be used.")
             return False
+        new_val = value.replace(' ', '').split(',')
+
+        # Even if one is valid, return true
+        for provider in new_val:
+            if provider in possM:
+                return True
+        return False
 
 
 def retDefault(keyword):
     """Return the DEFAULT value of keyword."""
     if keyword == 'QUALITY':
         return DEFAULTS().SONG_QUALITY
+    elif keyword == 'DEFAULT_FORMAT':
+        return DEFAULTS().DEFAULT_FORMAT
     elif keyword == 'SONG_DIR':
         return DEFAULTS().SONG_DIR
+    elif keyword == 'METADATA_PROVIDERS':
+        return DEFAULTS().METADATA_PROVIDERS
 
 
 def GIVE_DEFAULT(self, keyword):
@@ -233,14 +272,17 @@ def GIVE_DEFAULT(self, keyword):
 
                 # Remove the "
                 newDEFAULT = newDEFAULT.replace('"', '')
+                newDEFAULT = newDEFAULT.replace("'", '')
                 # Check if the line has a \n in it
                 if "\n" in line:
                     newDEFAULT = newDEFAULT.replace('\n', '')
 
-                if checkExistence(keyword, newDEFAULT):
+                if checkValidity(keyword, newDEFAULT):
                     return newDEFAULT
                 else:
-                    logger.warning("{}: doesn't exist.".format(newDEFAULT))
+                    if newDEFAULT:
+                        logger.warning(
+                            "{}: is invalid for option {}.".format(newDEFAULT, keyword))
                     return retDefault(keyword)
 
 
