@@ -4,25 +4,41 @@ All the functions used to interact with the downloaded song are defined here.
 """
 
 from colorama import Fore, Style
-from mutagen.id3 import ID3, APIC, TIT2, TPE1, TALB, TCON, TRCK, TYER
+from mutagen.id3 import (
+    ID3,
+    APIC,
+    TIT2,
+    TPE1,
+    TALB,
+    TCON,
+    TRCK,
+    TYER,
+    PictureType
+)
 from mutagen.mp3 import MP3
 from mutagen.mp4 import MP4, MP4Cover
+from mutagen import File
+from mutagen.flac import Picture
+from base64 import b64encode
 import requests
-from ytmdl import prepend, defaults, logger
 import os
+
+from ytmdl import prepend, defaults
+from simber import Logger
+from ytmdl.meta import preconfig
 # import traceback
 
-logger = logger.Logger("song")
+logger = Logger("song")
 
 # ----------------------cover--------------------
 
 
-def dwCover(SONG_INFO, index):
+def dwCover(song):
     """Download the song cover img from itunes."""
     # Try to download the cover art as cover.jpg in temp
     logger.info("Preparing the album cover")
     try:
-        imgURL = SONG_INFO[index].artwork_url_100
+        imgURL = song.artwork_url_100
 
         # Check if the passed imgURL is a local file
         # this is possible if the metadata was entered manually.
@@ -53,7 +69,8 @@ def dwCover(SONG_INFO, index):
         print('Could not get album cover. Are you connected to internet?\a')
         return False
     except Exception as e:
-        logger.warning("Error while trying to download image, skipping!: {}".format(e))
+        logger.warning(
+            "Error while trying to download image, skipping!: {}".format(e))
         return False
     else:
         return False
@@ -75,6 +92,7 @@ def print_choice(beg, end, SONG_INFO, type):
         if type == 'metadata':
             print(SONG_INFO[beg].track_name, end='')
         if type == 'mp3':
+            print('[M] ' if SONG_INFO[beg]['verified_music'] else '', end='')
             print(SONG_INFO[beg]['title'], end='')
         print(Style.RESET_ALL, end='')
         print(' by ', end='')
@@ -107,7 +125,9 @@ def getChoice(SONG_INFO, type):
     # In case less, print all
 
     prepend.PREPEND(1)
-    print('Choose One')
+    print('Choose One {}'.format(
+                        '(One with [M] is verified music)'
+                        if type == 'mp3' else ''))
 
     results = len(SONG_INFO)
 
@@ -122,7 +142,9 @@ def getChoice(SONG_INFO, type):
         if PRINT_WHOLE:
             print_choice(beg, results, SONG_INFO, type)
         prepend.PREPEND(1)
-        choice = input('Enter Choice [a valid choice] ')
+        choice = input('Enter Choice [default is 1] ')
+        if not choice:
+            choice = 1
         choice = int(choice)
         # If the choice is 6 then try to print more results
         if choice <= results and choice > 0:
@@ -139,7 +161,7 @@ def getChoice(SONG_INFO, type):
     return choice
 
 
-def set_MP3_data(SONG_INFO, is_quiet, song_path, choice):
+def set_MP3_data(song, song_path):
     """
     Set the meta data if the passed data is mp3.
     """
@@ -147,14 +169,6 @@ def set_MP3_data(SONG_INFO, is_quiet, song_path, choice):
     IS_IMG_ADDED = False
 
     try:
-        # If more than one choice then call getChoice
-        option = 0
-        if len(SONG_INFO) > 1:
-            if not is_quiet:
-                option = getChoice(SONG_INFO, 'metadata')
-            elif choice is not None and choice in range(1, len(SONG_INFO)):
-                option = choice
-
         SONG_PATH = os.path.join(defaults.DEFAULT.SONG_TEMP_DIR,
                                  song_path)
 
@@ -162,7 +176,7 @@ def set_MP3_data(SONG_INFO, is_quiet, song_path, choice):
         data = ID3(SONG_PATH)
 
         # Download the cover image, if failed, pass
-        if dwCover(SONG_INFO, option):
+        if dwCover(song):
             imagedata = open(defaults.DEFAULT.COVER_IMG, 'rb').read()
             data.add(APIC(3, 'image/jpeg', 3, 'Front cover', imagedata))
             # REmove the image
@@ -177,59 +191,48 @@ def set_MP3_data(SONG_INFO, is_quiet, song_path, choice):
 
         audio.save()
 
-        option = int(option)
-
-        data.add(TYER(encoding=3, text=SONG_INFO[option].release_date))
-        data.add(TIT2(encoding=3, text=SONG_INFO[option].track_name))
-        data.add(TPE1(encoding=3, text=SONG_INFO[option].artist_name))
-        data.add(TALB(encoding=3, text=SONG_INFO[option].collection_name))
-        data.add(TCON(encoding=3, text=SONG_INFO[option].primary_genre_name))
-        data.add(TRCK(encoding=3, text=str(SONG_INFO[option].track_number)))
+        data.add(TYER(encoding=3, text=song.release_date))
+        data.add(TIT2(encoding=3, text=song.track_name))
+        data.add(TPE1(encoding=3, text=song.artist_name))
+        data.add(TALB(encoding=3, text=song.collection_name))
+        data.add(TCON(encoding=3, text=song.primary_genre_name))
+        data.add(TRCK(encoding=3, text=str(song.track_number)))
 
         data.save()
 
-        defaults.DEFAULT.SONG_NAME_TO_SAVE = SONG_INFO[option].track_name + '.mp3'
+        defaults.DEFAULT.SONG_NAME_TO_SAVE = song.track_name + '.mp3'
 
         # Rename the downloaded file
         os.rename(SONG_PATH, os.path.join(
-                                    defaults.DEFAULT.SONG_TEMP_DIR,
-                                    defaults.DEFAULT.SONG_NAME_TO_SAVE
-                                ))
+            defaults.DEFAULT.SONG_TEMP_DIR,
+            defaults.DEFAULT.SONG_NAME_TO_SAVE
+        ))
 
-        return option, IS_IMG_ADDED
+        return IS_IMG_ADDED
 
     except Exception as e:
         logger.debug("{}".format(e))
         return e, False
 
 
-def set_M4A_data(SONG_INFO, is_quiet, song_path, choice):
+def set_M4A_data(song, song_path):
     """
     Set the tags in the m4a file passed.
     """
     cover_added = False
 
     try:
-        # If more than one choice then call getChoice
-        option = 0
-        if len(SONG_INFO) > 1:
-            if not is_quiet:
-                option = getChoice(SONG_INFO, 'metadata')
-            elif choice is not None and choice in range(1, len(SONG_INFO)):
-                option = choice
-
         SONG_PATH = os.path.join(defaults.DEFAULT.SONG_TEMP_DIR,
                                  song_path)
-
         audio = MP4(SONG_PATH)
 
         # Download the cover image, if failed, pass
-        if dwCover(SONG_INFO, option):
+        if dwCover(song):
             imagedata = open(defaults.DEFAULT.COVER_IMG, 'rb').read()
             audio["covr"] = [MP4Cover(
-                                imagedata,
-                                imageformat=MP4Cover.FORMAT_JPEG
-                            )]
+                imagedata,
+                imageformat=MP4Cover.FORMAT_JPEG
+            )]
             # REmove the image
             os.remove(defaults.DEFAULT.COVER_IMG)
             cover_added = True
@@ -242,61 +245,149 @@ def set_M4A_data(SONG_INFO, is_quiet, song_path, choice):
 
         audio.save()
 
-        option = int(option)
-
         # Add the meta data, the key's can be found at
         # https://mutagen.readthedocs.io/en/latest/api/mp4.html#mutagen.mp4.MP4Tags
-        audio["\xa9nam"] = SONG_INFO[option].track_name
-        audio["\xa9alb"] = SONG_INFO[option].collection_name
-        audio["\xa9ART"] = SONG_INFO[option].artist_name
-        audio["\xa9day"] = SONG_INFO[option].release_date
-        audio["\xa9gen"] = SONG_INFO[option].primary_genre_name
+        audio["\xa9nam"] = song.track_name
+        audio["\xa9alb"] = song.collection_name
+        audio["\xa9ART"] = song.artist_name
+        audio["\xa9day"] = song.release_date
+        audio["\xa9gen"] = song.primary_genre_name
 
         # Adding track number would probably thwor some kind
         # of render error, will leave for later
 
         audio.save()
 
-        defaults.DEFAULT.SONG_NAME_TO_SAVE = SONG_INFO[option].track_name + '.m4a'
+        defaults.DEFAULT.SONG_NAME_TO_SAVE = song.track_name + '.m4a'
 
         # Rename the downloaded file
         os.rename(SONG_PATH, os.path.join(
-                                    defaults.DEFAULT.SONG_TEMP_DIR,
-                                    defaults.DEFAULT.SONG_NAME_TO_SAVE
-                                ))
+            defaults.DEFAULT.SONG_TEMP_DIR,
+            defaults.DEFAULT.SONG_NAME_TO_SAVE
+        ))
 
-        return option, cover_added
+        return cover_added
 
     except Exception as e:
         return e
 
 
+def set_OPUS_data(song, song_path):
+    """
+    Set the data into an OPUS container according to the
+    passed data.
+    """
+    COVER_ADDED = False
+
+    try:
+        SONG_PATH = os.path.join(defaults.DEFAULT.SONG_TEMP_DIR,
+                                 song_path)
+        mutagen_file = File(SONG_PATH)
+
+        # Try adding the tags container
+        try:
+            mutagen_file.add_tags()
+        except Exception:
+            # If exception is thrown, the tags already exist
+            pass
+
+        # Clear out the tags from the file
+        mutagen_file.clear()
+
+        # Try adding the cover
+        if dwCover(song):
+            imagedata = open(defaults.DEFAULT.COVER_IMG, 'rb').read()
+            picture = Picture()
+            picture.data = imagedata
+            picture.type = PictureType.COVER_FRONT
+            picture.mime = "image/jpeg"
+            encoded_data = b64encode(picture.write())
+            mutagen_file["metadata_block_picture"] = encoded_data.decode(
+                                                     "ascii")
+
+            # Remove the image
+            os.remove(defaults.DEFAULT.COVER_IMG)
+            COVER_ADDED = True
+
+        # Add the tags now
+        # Refer to https://www.programcreek.com/python/example/63675/mutagen.File
+        # for more information on it
+        mutagen_file["Title"] = song.track_name
+        mutagen_file["Album"] = song.collection_name
+        mutagen_file["Artist"] = song.artist_name
+        mutagen_file["Date"] = song.release_date
+        mutagen_file["Genre"] = song.primary_genre_name
+
+        mutagen_file.save()
+
+        defaults.DEFAULT.SONG_NAME_TO_SAVE = song.track_name + '.opus'
+
+        # Rename the downloaded file
+        os.rename(SONG_PATH, os.path.join(
+            defaults.DEFAULT.SONG_TEMP_DIR,
+            defaults.DEFAULT.SONG_NAME_TO_SAVE
+        ))
+
+        return COVER_ADDED
+    except Exception as e:
+        return e
+
+
+def _get_option(SONG_INFO, is_quiet, choice):
+    option = 0
+    if len(SONG_INFO) > 1:
+        if not is_quiet:
+            option = getChoice(SONG_INFO, 'metadata')
+        elif choice is not None and choice in range(1, len(SONG_INFO)):
+            option = choice
+    return int(option)
+
+
 def setData(SONG_INFO, is_quiet, song_path, datatype='mp3', choice=None):
     """Add the metadata to the song."""
+
+    # Some providers need extra daa from other endpoints,
+    # this is where we define which need it and where to get
+    # it from
+
+    option = _get_option(SONG_INFO, is_quiet, choice)
+
+    song = SONG_INFO[option]
+
+    get_more_data_dict = preconfig.CONFIG().GET_EXTRA_DATA
+
+    # Try to check if the song object has an attribute provider
+    # Deezer has it but other objects don't have it.
+    # If the provider is present then fetch extra data accordingly
+
+    if hasattr(song, 'provider') and song.provider in get_more_data_dict:
+        song = get_more_data_dict.get(song.provider, lambda _: None)(song)
+
     if datatype == 'mp3':
-        option, img_added = set_MP3_data(
-                                SONG_INFO,
-                                is_quiet,
-                                song_path,
-                                choice
-                            )
+        img_added = set_MP3_data(
+            song,
+            song_path,
+        )
     elif datatype == 'm4a':
-        option, img_added = set_M4A_data(
-                                SONG_INFO,
-                                is_quiet,
-                                song_path,
-                                choice
-                            )
+        img_added = set_M4A_data(
+            song,
+            song_path,
+        )
+    elif datatype == 'opus':
+        img_added = set_OPUS_data(
+            song,
+            song_path
+        )
 
     # Show the written stuff in a better format
     prepend.PREPEND(1)
     print('================================')
-    print('  || YEAR: ' + SONG_INFO[option].release_date)
-    print('  || TITLE: ' + SONG_INFO[option].track_name)
-    print('  || ARTIST: ' + SONG_INFO[option].artist_name)
-    print('  || ALBUM: ' + SONG_INFO[option].collection_name)
-    print('  || GENRE: ' + SONG_INFO[option].primary_genre_name)
-    print('  || TRACK NO: ' + str(SONG_INFO[option].track_number))
+    print('  || YEAR: ' + song.release_date)
+    print('  || TITLE: ' + song.track_name)
+    print('  || ARTIST: ' + song.artist_name)
+    print('  || ALBUM: ' + song.collection_name)
+    print('  || GENRE: ' + song.primary_genre_name)
+    print('  || TRACK NO: ' + str(song.track_number))
 
     if img_added:
         print('  || ALBUM COVER ADDED')
