@@ -1,16 +1,17 @@
+
 """Definition of functions that are used to interact with youtube."""
 
 import requests
 from bs4 import BeautifulSoup
 import os
 import youtube_dl
-import re
+from re import match
 from ytmdl import defaults, utility, stringutils
 from downloader_cli.download import Download
 import traceback
 from sys import stdout
-
-from ytmdl.logger import Logger
+from youtube_search import YoutubeSearch
+from simber import Logger
 
 
 logger = Logger("yt")
@@ -69,14 +70,14 @@ def progress_handler(d):
         stdout.flush()
 
 
-def dw_using_yt(link, proxy, song_name, datatype):
+def dw_using_yt(link, proxy, song_name, datatype, no_progress=False):
     """
     Download the song using YTDL downloader and use downloader CLI's
     functions to be used to display a progressbar.
 
     The function will be called by using hooks.
     """
-    if datatype == 'mp3':
+    if datatype == 'mp3' or datatype == 'opus':
         format_ = 'bestaudio/best'
     elif datatype == 'm4a':
         format_ = 'bestaudio[ext=m4a]'
@@ -86,9 +87,12 @@ def dw_using_yt(link, proxy, song_name, datatype):
         'outtmpl': song_name,
         'format': format_,
         'nocheckcertificate': True,
-        'progress_hooks': [progress_handler],
+        'source_address': '0.0.0.0'
     }
 
+    if not no_progress:
+        ydl_opts['progress_hooks'] = [progress_handler]
+
     if proxy is not None:
         ydl_opts['proxy'] = proxy
 
@@ -102,7 +106,13 @@ def dw_using_yt(link, proxy, song_name, datatype):
         return e
 
 
-def dw(value, proxy=None, song_name='ytmdl_temp.mp3', datatype='mp3'):
+def dw(
+        value,
+        proxy=None,
+        song_name='ytmdl_temp.mp3',
+        datatype='mp3',
+        no_progress=False
+        ):
     """
     Download the song.
 
@@ -134,7 +144,7 @@ def dw(value, proxy=None, song_name='ytmdl_temp.mp3', datatype='mp3'):
         logger.debug(name)
 
         # Start downloading the song
-        status = dw_using_yt(value, proxy, name, datatype)
+        status = dw_using_yt(value, proxy, name, datatype, no_progress)
 
         if status == 0:
             return name
@@ -155,18 +165,28 @@ def get_href(url):
     return part
 
 
+def _is_verified(desc: str) -> bool:
+    """Determine if the music video is provided
+    by the music company to YouTube or not.
+
+    In other words, this will just indicate if the
+    song is directly provided by the music producer.
+
+    This can be determined in a few ways.
+
+    - If the desc contains `Auto-Generated by YouTube`
+    - If the desc contains `Provided to YouTube by <company>`
+    """
+    return bool(match(r'^provided.to.youtube.by.*|^auto.generated.by.youtube', desc.lower()))
+
+
 def search(query, bettersearch, proxy, kw=[], lim=20):
     """
-    Search the video on youtube using `youtube-scrape`.
+    Search the song name and return results
 
-    Youtube Scrape is a project that exposes an URL deployed
-    on Heroku that can be used to make calls to Youtube's API.
-
-    Go say thanks to them at https://github.com/HermanFassett/youtube-scrape
+    Search the passed query using the `youtube_search` module
+    and extract the results accordingly and return.
     """
-    # Base API url
-    BASE_URL = 'http://youtube-scrape.herokuapp.com/api/search'
-
     # Add keywords if better search is enabled
     kw = [kw_ for kw_ in kw if kw_ is not None]
 
@@ -181,120 +201,29 @@ def search(query, bettersearch, proxy, kw=[], lim=20):
     # Replace all the spaces with +
     query = query.replace(' ', '+')
 
-    url = "{}?q={}".format(BASE_URL, query)
+    results = YoutubeSearch(query).to_dict()
 
-    try:
-        response = requests.get(url, proxies=proxies)
-    except requests.exceptions.ConnectionError:
-        logger.critical("Connection Error! Are you connected to internet?")
-    except requests.exceptions.Timeout:
-        logger.critical("Timed Out! Are you connected to internet?")
-    except Exception:
-        traceback.print_exc()
-
-    # Check if everythin was ok
-    if response.status_code != 200:
-        logger.critical("Was not able to search the video")
-
-    response = response.json()
-
-    try:
-        results = response['results']
-    except KeyError:
-        logger.critical(
-                    "No result was returned. Returned data is: {}".format(
-                                                                    response
-                                                                )
-                )
-
-    # Check if any result was even found.
-    if not len(results):
-        logger.critical("No results were found. Try a different keyword?!")
+    if not results:
+        logger.warning("No search results found!")
 
     # Trim the data to the passed number
     if len(results) > lim:
         results = results[:lim]
 
-    extracted_data = []
+    stripped_results = []
 
     for video in results:
-        # Put a check to see the result is not a channel
-        if 'video' not in list(video.keys()):
-            continue
-
         data = {}
-        data['title'] = video['video']['title']
-        data['href'] = video['video']['url'].replace(
-                                                'https://www.youtube.com',
-                                                ''
-                                            )
-        data['author_name'] = video['uploader']['username']
-        data['duration'] = video['video']['duration']
+        data['title'] = video['title']
+        data['href'] = video['url_suffix']
+        data['author_name'] = video['channel']
+        data['duration'] = video['duration']
+        data['verified_music'] = False if video['long_desc'] is None else \
+            _is_verified(video['long_desc'])
 
-        extracted_data.append(data)
+        stripped_results.append(data)
 
-    return extracted_data
-
-
-def search2(query, bettersearch, proxy, kw=[], lim=10):
-    """Search the query in youtube and return lim number of results.
-
-    Query is the keyword, i:e name of the song
-    lim is the number of songs that will be added to video array and returned
-    """
-
-    # Add keywords if better search is enabled
-    if bettersearch:
-        for keyword in kw:
-            if keyword is not None:
-                query += ' ' + keyword
-
-    # Check if proxy is passed.
-    proxies = {}
-    if proxy is not None:
-        proxies['http'] = proxy
-
-    # Replace all the spaces with +
-    query = query.replace(' ', '+')
-
-    url = "https://www.youtube.com/results?search_query={}".format(query)
-    videos = []
-
-    try:
-        response = requests.get(url, proxies=proxies)
-        soup = BeautifulSoup(response.text, "lxml")
-        videos = soup.findAll('div', attrs={'class': 'yt-lockup-content'})
-    except requests.exceptions.ConnectionError:
-        logger.critical("Connection Error! Are you connected to internet?")
-    except requests.exceptions.Timeout:
-        logger.critical("Timed Out! Are you connected to internet?")
-    except Exception:
-        traceback.print_exc()
-
-    if not videos:
-        return []
-
-    if len(videos) > lim:
-        videos = videos[:lim]
-
-    extracted_data = []
-
-    for video in videos:
-        a = video.find_all('a')
-        # This check is necessary because in some cases the search results
-        # contain channel names etc.
-        if len(a) <= 1: continue
-        data = {}
-        data['title'] = a[0]['title']
-        data['href'] = a[0]['href']
-        data['author_name'] = a[1].text
-        duration_unprocessed = video.span.text
-        duration = re.sub(r'\ |\-|\.|Duration', '', duration_unprocessed)
-        data['duration'] = re.subn(r':', '', duration, 1)[0]
-
-        extracted_data.append(data)
-
-    return extracted_data
+    return stripped_results
 
 
 def scan_video(url, proxy):
@@ -386,7 +315,14 @@ def get_title(url):
     """
     Return the title of the passed URL.
     """
-    ydl_opts = {"quiet": True}
+    ydl_opts = {
+            "quiet": True,
+            'nocheckcertificate': True,
+            'source_address': '0.0.0.0'
+        }
+
+    logger.debug(url)
+
     ydl = youtube_dl.YoutubeDL(ydl_opts)
     data = ydl.extract_info(url, False)
 
